; Multiface Two ROM Dump utility
; www.grimware.org

_KL_LROM_ENABLE                 equ &B906
_KL_ROM_RESTORE                 equ &B90C
_KM_WAIT_KEY                    equ &BB18
_TXT_OUTPUT                     equ &BB5A

                                org &8000

                                ; Display some propaganda
                                ld hl,_textdata_boot
                                call _print_ntstring
                                ; enable the lower ROM
                                call _KL_LROM_ENABLE
                                push af
                                ; Ignore interrupts
                                di
                                ; page in the MF2 ROM
                                ; &0000-&1FFF (8Kb)
                                ld bc,&FEE8
                                out (c),c
                                ; Check if the MF2 is available
                                ld a,(&38)
                                cp &C3
                                jr z,_mf2_not_found
                                ; Copy the ROM to &4000
                                ld hl,&0000
                                ld de,&4000
                                ld bc,&2000
                                ldir
                                ; page out the MF2 ROM
                                ld bc,&FEEA
                                out (c),c
                                ; Acknowledge Interrupts
                                ei
                                ; disable the lower ROM
                                pop af
                                call _KL_ROM_RESTORE
                                ; Display a message
                                ld hl,_textdata_saving_rom
                                call _print_ntstring
                                ; Save file to disc
                                call amsdos_save
                                dw _filename
                                db 7
                                dw &6000
                                dw &4000
                                dw &2000
                                dw &DEAD
                                db 2
                                ; Display another message
                                ld hl,_textdata_exit
                                call _print_ntstring
                                ; Wait a key and reset
                                jr _mf2_wkrst

_mf2_not_found                  ; disable the lower ROM
                                pop af
                                call _KL_ROM_RESTORE
                                ; Display an error message
                                ld hl,_textdata_mf2_notfound
                                call _print_ntstring
_mf2_wkrst                      ; Wait a key and reset
                                call _KM_WAIT_KEY
                                rst 0

;*******************************************************************************

_print_ntstring:
                                ld a,(hl)
                                or a
                                ret z
                                call _TXT_OUTPUT
                                inc hl
                                jr _print_ntstring

_textdata_boot                  db 4,1, 28,1,13,13, 28,2,18,18
                                db 15,1,"Multiface Two ROM Dump utility",&A,&D
                                db "from http://www.grimware.org/",&A,&D
                                db &A,&D, 0

_textdata_mf2_notfound          db 15,3, 28,3,6,6, "No Multiface Two found!",&A,&D
                                db 15,1,"Press any key to reset.",0

_textdata_saving_rom            db 15,2,"Multiface Two ROM found!",&A,&D, 28,3,26,26
                                db 15,1,"Saving ",15,3
_filename                       db "MF2.ROM"
                                db 15,1," file to disc...", 0

_textdata_exit                  db 15,2,"Done!",&A,&D,&A,&D
                                db 15,3,"Dump this disk and send the DSK to:",&A,&D
                                db "grim(at)cpcscene.com",&A,&D,&A,&D
                                db "Thank you for your support!",0

                                ; Usage
                                ; call amsdos_save
                                ; dw filename_adr
                                ; db filename_lenght
                                ; dw 2kb buffer address
                                ; dw start_address
                                ; dw file_lenght
                                ; dw exec_address
                                ; db file_type
amsdos_save:
                                pop ix
                                ld h,(ix+1)     ; filename address
                                ld l,(ix+0)
                                ld b,(ix+2)     ; filename lenght
                                ld d,(ix+4)     ; 2kb buffer
                                ld e,(ix+3)
                                call &BC8C

                                ld h,(ix+6)     ; save_adr
                                ld l,(ix+5)
                                ld d,(ix+8)     ; save_lenght
                                ld e,(ix+7)
                                ld b,(ix+10)    ; file_exec
                                ld c,(ix+9)
                                ld a,(ix+11)    ; file_type
                                call &BC98
                                call &BC8F
                                call &BC7D

                                ld bc,12
                                add ix,bc
                                jp (ix)
